# Author: Acer Zhang
# Datetime: 2021/9/28 
# Copyright belongs to the author.
# Please indicate the source for reprinting.
import os
import re
import requests

from qgui import CreateQGUI
from qgui.banner_tools import GitHub
from qgui.notebook_tools import *
from qwebsite.base import GitHubOptimizer
from qwebsite.flags import *

from qwebsite import __version__

ISO_LIST = ["https://hub.fastgit.org",
            "https://github.com.cnpmjs.org",
            "https://git.sdut.me",
            "https://github.wuyanzheshui.workers.dev"]


def set_iso(url):
    out = os.popen(f"git config --global url.{url}.insteadof https://github.com/").read()
    if out:
        print("执行情况如下：\n", out)


def ping_iso(url):
    print(f"开始测试{url}镜像源")
    try:
        ping_time = requests.get(url, timeout=4).elapsed.total_seconds()
        if ping_time < 1:
            print(f"镜像源\t{url}网络质量较好")
        elif ping_time < 2:
            print(f"镜像源\t{url}网络质量一般")
        else:
            print(f"镜像源\t{url}网络质量较差")
        print(f"镜像源\t{url}的响应时间为{ping_time:.4f}s")
    except requests.exceptions.ConnectTimeout:
        print(f"镜像源\t{url}长时间无响应")




def clean_iso(url):
    out = os.popen(f"git config --global --unset url.{url}.insteadof").read()
    if out:
        print("执行情况如下：\n", out)


def get_iso():
    out = os.popen(f"git config --global -l").read()
    url = re.findall(".*url\.(.*)\.insteadof.*", out)
    if url:
        url = url[0]
    else:
        url = "GitHub官方"
    return url


def ping_callback(args):
    url = args["下拉选择框"].get()
    ping_iso(url=url)


def clean_callback(args):
    clean_iso(get_iso())
    args["当前镜像源情况"].set(get_iso())
    print("清空完毕")


def set_callback(args):
    url = args["下拉选择框"].get()
    set_iso(url)
    args["当前镜像源情况"].set(get_iso())
    print("设置完毕")


def set_hosts(args):
    GitHubOptimizer()


def reset_hosts(args):
    GitHubOptimizer(RESET_FLAG)


class QWebSiteOptGUI:
    def __init__(self, title="QWebSiteOptimizer - GitHub"):
        # 基础GUI
        self.gui = CreateQGUI(title=title)
        self.gui.add_banner_tool(GitHub("https://github.com/QPT-Family/QWebSiteOptimizer"))
        self.gui.set_navigation_about(author="GT_ZhangAcer",
                                      version=__version__,
                                      github_url="https://github.com/QPT-Family/QWebSiteOptimizer")
        # DNS解析选优功能区
        set_dns_opt = BaseButton(bind_func=set_hosts, text="开始DNS选优", style="info")
        clear_dns_opt = BaseButton(bind_func=reset_hosts, text="清除DNS选优", style="success")
        self.gui.add_notebook_tool(HorizontalToolsCombine(tools=[set_dns_opt, clear_dns_opt],
                                                          title="[ROOT] DNS解析选优 - 需要管理员权限",
                                                          text="通过对多个DNS返回的结果进行筛选，匹配当前网络环境下最快的解析地址并设置Host文件，"
                                                               "从而达到加速目的。\n"
                                                               "该方式虽然不如VPN稳定可靠，但在短期内我们可通过该加速效果来达到优化GitHub浏览速度等问题。\n"
                                                               "建议在阅览GitHub出现卡顿时重新进行设置，若需要Clone大型项目，可切换至镜像源模式进行解决。"))

        # 镜像源模式功能区
        act = get_iso()
        if not act:
            act = "GitHub官方"
        iso_act = Label(name="当前镜像源情况", text=act, title="当前镜像源:")
        iso_comb = Combobox(name="下拉选择框", title="选择镜像源:", options=ISO_LIST)

        iso_ping = BaseButton(bind_func=ping_callback, text="Ping测试", style="info")
        iso_clear = BaseButton(bind_func=clean_callback, text="清除镜像源", style="success")
        iso_set = BaseButton(bind_func=set_callback, text="设置镜像源", style="danger")

        iso_btn = HorizontalToolsCombine(tools=[iso_comb, iso_ping, iso_clear, iso_set])

        self.gui.add_notebook_tool(HorizontalFrameCombine(tools=[iso_act, iso_btn],
                                                          title="镜像源模式",
                                                          text="通过对GitHub默认源进行替换，从而快速从镜像源中加载我们需要的数据，提升Clone的效率。\n"
                                                               "但由于镜像源并非官方源，强烈、非常强烈建议我们仅在需要Clone公开项目时开启该模式，"
                                                               "在Push以及私有项目的操作中保持关闭状态，避免出现异常情况！"))

    def run(self):
        self.gui.run()


if __name__ == '__main__':
    # _u = "https://hub.fastgit.org"
    # set_iso(_u)
    # _a = get_iso()
    # print(_a)
    # clean_iso(_a[0])
    # _b = get_iso()
    # print("b:", _b)
    _a = QWebSiteOptGUI()
    _a.run()



import socket
import time
import sys
import requests

from qwebsite.flags import *


def get_active_ip(host="github.com"):
    ips = list()
    try:
        result = socket.getaddrinfo(host, None, socket.AddressFamily.AF_INET, socket.SOCK_STREAM)
    except socket.gaierror:
        return ips

    for item in result:
        ip = item[4][0]
        try:
            ping_time = requests.get(ip, timeout=4).elapsed.total_seconds()
            ips.append((ip, ping_time))
        except requests.exceptions.ConnectTimeout:
            pass
    ips_sort = sorted(ips, key=lambda x: x[1])
    return ips_sort


class EditHost:
    def __init__(self, hosts_file_path=None):
        if hosts_file_path is None:
            hosts_file_path = HOSTS_FILE_PATH
        self.hosts_file_path = hosts_file_path
        # 记录额外信息
        self.hosts_data = list()
        self.hosts_data_end = list()
        # 记录KV 地址-IP
        self.host_kv = dict()
        self._update()

    def add_data(self, ip, address):
        self.host_kv[address] = ip

    def del_data(self, address):
        if address in self.host_kv:
            self.host_kv.pop(address)

    def _update(self):
        self.hosts_data = list()
        self.host_kv = dict()
        with open(self.hosts_file_path, "r") as f:
            hosts_data = f.readlines()
        for line_ori in hosts_data:
            line = line_ori.strip("\n").split("\t")
            if len(line) == 2:
                self.host_kv[line[1]] = line[0]
            elif not line[0]:
                continue
            elif line[0][0] == "#":
                self.hosts_data.append(line[0] + "\n")
            else:
                self.hosts_data.append(line_ori)

    def write(self, out_dir=None):
        if not out_dir:
            out_dir = self.hosts_file_path
        pre_data = self.hosts_data.copy()
        for k, v in self.host_kv.items():
            line = f"{v}\t{k}\n"
            pre_data.append(line)
        with open(out_dir, "w") as f:
            f.writelines(pre_data + self.hosts_data_end)
        self._update()


class BaseOptimizer:
    def __init__(self, urls, mode=ADD_FLAG, progressbar=None):
        if OS_FLAG == OS_FLAG_WINDOWS:
            try:
                import ctypes
                ctypes.windll.shell32.ShellExecuteW(None, "runas", sys.executable, __file__, None, 1)
            except Exception as e:
                print("当前操作系统安全策略无法使用ctypes模块，请安装杀毒软件修复系统漏洞进行解决，报错如下：", e)
                exit(1)
        self.urls = urls
        self.ed = EditHost()
        self.mode = mode
        self._make()
        self.print_kv()
        if progressbar:
            self.progressbar = progressbar
        else:
            class TMP:
                def __init__(self):
                    pass
                def set(self,n):
                    pass
            self.progressbar = TMP()

    def _make(self):
        scale = 1. / len(self.urls)
        if self.mode == RESET_FLAG:
            for url in self.urls:
                self.ed.del_data(url)
        else:
            os.popen('ipconfig /flushdns')
            for url_id, url in enumerate(self.urls):
                # self.progressbar.set(scale * scale)
                for i in range(2):
                    ip_info = get_active_ip(url)
                    if ip_info:
                        ip, ms = ip_info[0]
                        print(f"Info:\t{url}\t匹配到最快路径，延时为{ms:.4f}ms")
                        self.ed.add_data(ip, url)
                        break
                    else:
                        print(f"\r正在尝试重连...第{i + 1}次", flush=True, end="")
                        time.sleep(3)
                else:
                    print(f"\r\033[33mWarning:\t{url}\t中未搜索到可用IP，可能由于政策与法规限制，也可能是您的DNS出现了问题，"
                          f"可尝试修改本地网络DNS设置来解决非政策引起的搜索失败问题。\033[0m")
        self.ed.write()
        # self.progressbar.set(100)

    def print_kv(self):
        print("# -----------以下为当前网络环境下站点匹配情况-----------")
        kv = self.ed.host_kv
        for k, v in kv.items():
            line = f"{v}\t{k}\n"
            print(line)
        print("# -----------以下为当前网络环境下站点匹配情况-----------")


class GitHubOptimizer(BaseOptimizer):
    def __init__(self, mode=ADD_FLAG):
        # 此处URL来源整理自 GitHub:521xueweihan/GitHub520
        urls = ["alive.github.com",
                "live.github.com",
                "github.githubassets.com",
                "central.github.com",
                "desktop.githubusercontent.com",
                "assets-cdn.github.com",
                "camo.githubusercontent.com",
                "github.map.fastly.net",
                "github.global.ssl.fastly.net",
                "gist.github.com",
                "github.io",
                "github.com",
                "github.blog",
                "api.github.com",
                "raw.githubusercontent.com",
                "user-images.githubusercontent.com",
                "favicons.githubusercontent.com",
                "avatars5.githubusercontent.com",
                "avatars4.githubusercontent.com",
                "avatars3.githubusercontent.com",
                "avatars2.githubusercontent.com",
                "avatars1.githubusercontent.com",
                "avatars0.githubusercontent.com",
                "avatars.githubusercontent.com",
                "github-cloud.s3.amazonaws.com",
                "github-com.s3.amazonaws.com",
                "github-production-release-asset-2e65be.s3.amazonaws.com",
                "github-production-user-asset-6210df.s3.amazonaws.com",
                "github-production-repository-file-5c1aeb.s3.amazonaws.com",
                "githubstatus.com",
                "github.community",
                "media.githubusercontent.com"]
        super(GitHubOptimizer, self).__init__(urls, mode)


if __name__ == '__main__':
    # print(get_active_ip("www.baidu.com"))

    # _ed = EditHost()
    # _url = "www.baidu.com"
    # _tmp = get_active_ip("www.baidu.com")[0][0]
    # _ed.add_data(_tmp, _url)
    # _ed.write("./test.txt")
    GitHubOptimizer()
    # https://www.cnblogs.com/nicholas-920610/articles/7149057.html



# Author: Acer Zhang
# Datetime: 2021/9/8 
# Copyright belongs to the author.
# Please indicate the source for reprinting.

from qwebsite.flags import *

print("当前操作系统：", OS_FLAG)

__version__ = "0.4a1"



# Author: Acer Zhang
# Datetime: 2021/9/8 
# Copyright belongs to the author.
# Please indicate the source for reprinting.
import platform
import os

OS_FLAG_WINDOWS = "Windows"
OS_FLAG_MAC = "Darwin"
OS_FLAG_LINUX = "Linux"

OS_FLAG = platform.system()

HOSTS_FILE_PATH = None


if OS_FLAG == OS_FLAG_LINUX or OS_FLAG == OS_FLAG_MAC:
    HOSTS_FILE_PATH = "/etc/hosts"
else:
    HOSTS_FILE_PATH = os.path.join(os.environ["SYSTEMROOT"], "System32/drivers/etc/hosts")


ADD_FLAG = "ADD"
RESET_FLAG = "RESET"



# Author: Acer Zhang
# Datetime: 2021/9/8 
# Copyright belongs to the author.
# Please indicate the source for reprinting.

import click
from qwebsite.base import GitHubOptimizer
from qwebsite.flags import *
from qwebsite.submethod.github import QWebSiteOptGUI


@click.command()
@click.option("-g",
              "--gui",
              type=bool,
              default=True,
              help="图形化界面模式")
@click.option("-r",
              "--reset",
              type=bool,
              default=False,
              help="恢复模式")
def cli(gui, reset):
    if gui:
        QWebSiteOptGUI().run()
    if reset:
        GitHubOptimizer(mode=RESET_FLAG)
    else:
        GitHubOptimizer()


if __name__ == '__main__':
    cli([])



# Author: Acer Zhang
# Datetime: 2021/9/8 
# Copyright belongs to the author.
# Please indicate the source for reprinting.

from setuptools import setup
from setuptools import find_packages
from qwebsite import __version__

# python setup.py sdist bdist_wheel
setup(
    name='QWebSite',
    version=__version__,
    packages=find_packages(),
    url='https://github.com/QPT-Family/QWebSiteOptimizer',
    license='LGPL',
    author='GT-ZhangAcer',
    author_email='zhangacer@foxmail.com',
    description='QWebSite',
    install_requires=["click",
                      "qgui"],
    python_requires='>3.5',
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'qwebsite = qwebsite.__main__:cli',
        ]}
)
